@@ -36,9 +36,7 @@ static void _integrity_display(const struct lv_segment *seg)
 }
 
 static int _integrity_text_import(struct lv_segment *seg,
-				   const struct dm_config_node *sn,
-				   struct dm_hash_table *pv_hash __attribute__((unused)),
-				   struct dm_hash_table *lv_hash)
+				   const struct dm_config_node *sn)
 {
 	struct integrity_settings *set;
 	struct logical_volume *origin_lv = NULL;
@@ -59,7 +57,7 @@ static int _integrity_text_import(struct lv_segment *seg,
 	if (!dm_config_get_str(sn, "origin", &origin_name))
 		return SEG_LOG_ERROR("origin must be a string in");
 
-	if (!(origin_lv = dm_hash_lookup(lv_hash, origin_name)))
+	if (!(origin_lv = find_lv(seg->lv->vg, origin_name)))
 		return SEG_LOG_ERROR("Unknown LV specified for integrity origin %s in", origin_name);
 
 	if (!set_lv_segment_area_lv(seg, 0, origin_lv, 0, 0))
@@ -104,7 +102,7 @@ static int _integrity_text_import(struct lv_segment *seg,
 		if (!dm_config_get_str(sn, "meta_dev", &meta_dev))
 			return SEG_LOG_ERROR("meta_dev must be a string in");
 
-		if (!(meta_lv = dm_hash_lookup(lv_hash, meta_dev)))
+		if (!(meta_lv = find_lv(seg->lv->vg, meta_dev)))
 			return SEG_LOG_ERROR("Unknown logical volume %s specified for integrity in", meta_dev);
 	}
 
