@@ -80,7 +80,7 @@ static uint64_t _lv_size_bytes_to_integrity_meta_bytes(uint64_t lv_size_bytes, u
 		initial_bytes = 8 * ONE_MB_IN_BYTES;
 	else if (lv_size_bytes <= (4ULL * ONE_GB_IN_BYTES))
 		initial_bytes = 32 * ONE_MB_IN_BYTES;
-	else if (lv_size_bytes > (4ULL * ONE_GB_IN_BYTES))
+	else // if (lv_size_bytes > (4ULL * ONE_GB_IN_BYTES))
 		initial_bytes = 64 * ONE_MB_IN_BYTES;
  out:
 	log_debug("integrity_meta_bytes %llu from lv_size_bytes %llu meta_bytes %llu initial_bytes %llu journal_sectors %u",
@@ -231,16 +231,17 @@ int lv_extend_integrity_in_raid(struct logical_volume *lv, struct dm_list *pvh)
 	return 1;
 }
 
-int lv_remove_integrity_from_raid(struct logical_volume *lv)
+int lv_remove_integrity_from_raid(struct logical_volume *lv, char **remove_images)
 {
-	struct logical_volume *iorig_lvs[DEFAULT_RAID_MAX_IMAGES];
-	struct logical_volume *imeta_lvs[DEFAULT_RAID_MAX_IMAGES];
+	struct logical_volume *iorig_lvs[DEFAULT_RAID_MAX_IMAGES] = { 0 };
+	struct logical_volume *imeta_lvs[DEFAULT_RAID_MAX_IMAGES] = { 0 };
 	struct cmd_context *cmd = lv->vg->cmd;
 	struct volume_group *vg = lv->vg;
 	struct lv_segment *seg_top, *seg_image;
 	struct logical_volume *lv_image;
 	struct logical_volume *lv_iorig;
 	struct logical_volume *lv_imeta;
+	char *max_image_array = remove_images ? *remove_images : NULL;
 	uint32_t area_count, s;
 	int is_active = lv_is_active(lv);
 
@@ -253,12 +254,19 @@ int lv_remove_integrity_from_raid(struct logical_volume *lv)
 		return 0;
 	}
 
+	log_debug("Removing integrity from raid LV %s %s", display_lvname(lv), remove_images ? "partial" : "");
+
 	area_count = seg_top->area_count;
 
 	for (s = 0; s < area_count; s++) {
+		if (max_image_array && !max_image_array[s])
+			continue;
+
 		lv_image = seg_lv(seg_top, s);
 		seg_image = first_seg(lv_image);
 
+		log_debug("Removing integrity layers from %s", lv_image->name);
+
 		if (!(lv_imeta = seg_image->integrity_meta_dev)) {
 			log_error("LV %s segment has no integrity metadata device.", display_lvname(lv));
 			return 0;
@@ -293,10 +301,15 @@ int lv_remove_integrity_from_raid(struct logical_volume *lv)
 	}
 
 	for (s = 0; s < area_count; s++) {
+		if (max_image_array && !max_image_array[s])
+			continue;
+
 		lv_iorig = iorig_lvs[s];
 		lv_imeta = imeta_lvs[s];
 
 		if (is_active) {
+			log_debug("Deactivating unused integrity layers %s %s", lv_iorig->name, lv_imeta->name);
+
 			if (!deactivate_lv(cmd, lv_iorig))
 				log_error("Failed to deactivate unused iorig LV %s.", lv_iorig->name);
 
@@ -307,6 +320,8 @@ int lv_remove_integrity_from_raid(struct logical_volume *lv)
 		lv_imeta->status &= ~INTEGRITY_METADATA;
 		lv_set_visible(lv_imeta);
 
+		log_debug("Removing unused integrity LVs %s %s", lv_iorig->name, lv_imeta->name);
+
 		if (!lv_remove(lv_iorig))
 			log_error("Failed to remove unused iorig LV %s.", lv_iorig->name);
 
