@@ -32,11 +32,10 @@
 //#define STRIPE_SIZE_MIN ( (unsigned) lvm_getpagesize() >> SECTOR_SHIFT)	/* PAGESIZE in sectors */
 //#define STRIPE_SIZE_MAX ( 512L * 1024L >> SECTOR_SHIFT)	/* 512 KB in sectors */
 //#define STRIPE_SIZE_LIMIT ((UINT_MAX >> 2) + 1)
-//#define MAX_RESTRICTED_LVS 255	/* Used by FMT_RESTRICTED_LVIDS */
 #define MIN_PE_SIZE     (8192L >> SECTOR_SHIFT) /* 8 KB in sectors - format1 only */
 #define MAX_PE_SIZE     (16L * 1024L * (1024L >> SECTOR_SHIFT) * 1024L) /* format1 only */
 #define MIRROR_LOG_OFFSET	2	/* sectors */
-#define VG_MEMPOOL_CHUNK	10240	/* in bytes, hint only */
+#define VG_MEMPOOL_CHUNK	63000	/* in bytes, hint only */
 
 /*
  * Ceiling(n / sz)
@@ -74,7 +73,6 @@ struct cached_vg_fmtdata;
 
 /* Per-format per-metadata area operations */
 struct metadata_area_ops {
-	struct dm_list list;
 	struct volume_group *(*vg_read) (struct cmd_context *cmd,
 					 struct format_instance * fi,
 					 const char *vg_name,
@@ -184,7 +182,7 @@ struct metadata_area_ops {
 
 struct metadata_area {
 	struct dm_list list;
-	struct metadata_area_ops *ops;
+	const struct metadata_area_ops *ops;
 	void *metadata_locn;
 	uint32_t status;
 	uint64_t header_start; /* mda_header.start */
@@ -224,7 +222,7 @@ struct format_instance *alloc_fid(const struct format_type *fmt,
 
 /*
  * Format instance must always be set using pv_set_fid or vg_set_fid
- * (NULL value as well), never asign it directly! This is essential
+ * (NULL value as well), never assign it directly! This is essential
  * for proper reference counting for the format instance.
  */
 void pv_set_fid(struct physical_volume *pv, struct format_instance *fid);
@@ -386,7 +384,7 @@ void set_pe_align_offset(struct physical_volume *pv, uint64_t data_alignment_off
 int pv_write_orphan(struct cmd_context *cmd, struct physical_volume *pv);
 
 int check_dev_block_size_for_vg(struct device *dev, const struct volume_group *vg,
-				unsigned int *max_phys_block_size_found);
+				unsigned int *max_logical_block_size_found);
 int check_pv_dev_sizes(struct volume_group *vg);
 uint32_t vg_bad_status_bits(const struct volume_group *vg, uint64_t status);
 int add_pv_to_vg(struct volume_group *vg, const char *pv_name,
@@ -397,6 +395,7 @@ struct logical_volume *find_lv_in_vg_by_lvid(const struct volume_group *vg,
 
 /* FIXME Merge these functions with ones above */
 struct physical_volume *find_pv(struct volume_group *vg, struct device *dev);
+struct physical_volume *find_pv_by_pv_name(struct volume_group *vg, const char *pv_name);
 
 struct pv_list *find_pv_in_pv_list(const struct dm_list *pl,
 				   const struct physical_volume *pv);
@@ -408,7 +407,7 @@ struct lv_segment *find_seg_by_le(const struct logical_volume *lv, uint32_t le);
 struct lv_segment *find_pool_seg(const struct lv_segment *seg);
 
 /* Find some unused device_id for thin pool LV segment. */
-uint32_t get_free_pool_device_id(struct lv_segment *thin_pool_seg);
+uint32_t get_free_thin_pool_device_id(struct lv_segment *thin_pool_seg);
 
 /* Check if the new thin-pool could be used for lvm2 thin volumes */
 int check_new_thin_pool(const struct logical_volume *pool_lv);
@@ -420,11 +419,10 @@ const char *strip_dir(const char *vg_name, const char *dev_dir);
 
 struct logical_volume *alloc_lv(struct dm_pool *mem);
 
-/*
- * Checks that an lv has no gaps or overlapping segments.
- * Set complete_vg to perform additional VG level checks.
- */
-int check_lv_segments(struct logical_volume *lv, int complete_vg);
+/* Checks that an lv has no gaps or overlapping segments. */
+int check_lv_segments_incomplete_vg(struct logical_volume *lv);
+/* Aditional VG level checks on lv segment. */
+int check_lv_segments_complete_vg(struct logical_volume *lv);
 
 /*
  * Does every LV segment have the same number of stripes?
@@ -433,7 +431,7 @@ int lv_has_constant_stripes(struct logical_volume *lv);
 
 /*
  * Sometimes (eg, after an lvextend), it is possible to merge two
- * adjacent segments into a single segment.  This function trys
+ * adjacent segments into a single segment.  This function tries
  * to merge as many segments as possible.
  */
 int lv_merge_segments(struct logical_volume *lv);
@@ -490,31 +488,15 @@ struct volume_group *vg_from_config_tree(struct cmd_context *cmd, const struct d
 int fixup_imported_mirrors(struct volume_group *vg);
 
 /*
- * From thin_manip.c
+ * From pool_manip.c
  */
 int attach_pool_lv(struct lv_segment *seg, struct logical_volume *pool_lv,
 		   struct logical_volume *origin,
 		   struct generic_logical_volume *indirect_origin,
 		   struct logical_volume *merge_lv);
 int detach_pool_lv(struct lv_segment *seg);
-int attach_pool_message(struct lv_segment *pool_seg, dm_thin_message_t type,
-			struct logical_volume *lv, uint32_t delete_id,
-			int no_update);
-int lv_is_merging_thin_snapshot(const struct logical_volume *lv);
-int pool_has_message(const struct lv_segment *seg,
-		     const struct logical_volume *lv, uint32_t device_id);
-int pool_metadata_min_threshold(const struct lv_segment *pool_seg);
-int pool_below_threshold(const struct lv_segment *pool_seg);
-int pool_check_overprovisioning(const struct logical_volume *lv);
 int create_pool(struct logical_volume *pool_lv, const struct segment_type *segtype,
 		struct alloc_handle *ah, uint32_t stripes, uint32_t stripe_size);
-uint64_t get_thin_pool_max_metadata_size(struct cmd_context *cmd, struct profile *profile,
-					 thin_crop_metadata_t *crop);
-thin_crop_metadata_t get_thin_pool_crop_metadata(struct cmd_context *cmd,
-						  thin_crop_metadata_t crop,
-						  uint64_t metadata_size);
-uint64_t estimate_thin_pool_metadata_size(uint32_t data_extents, uint32_t extent_size, uint32_t chunk_size);
-
 int update_pool_metadata_min_max(struct cmd_context *cmd,
 				 uint32_t extent_size,
 				 uint64_t min_metadata_size,		/* required min */
@@ -523,6 +505,25 @@ int update_pool_metadata_min_max(struct cmd_context *cmd,
 				 struct logical_volume *metadata_lv,	/* name of converted LV or NULL */
 				 uint32_t *metadata_extents);		/* resulting extent count */
 
+/*
+ * From thin_manip.c
+ */
+int attach_thin_pool_message(struct lv_segment *pool_seg, dm_thin_message_t type,
+			     struct logical_volume *lv, uint32_t delete_id,
+			     int no_update);
+int lv_is_merging_thin_snapshot(const struct logical_volume *lv);
+int thin_pool_has_message(const struct lv_segment *seg,
+			  const struct logical_volume *lv, uint32_t device_id);
+int thin_pool_metadata_min_threshold(const struct lv_segment *pool_seg);
+int thin_pool_below_threshold(const struct lv_segment *pool_seg);
+int thin_pool_check_overprovisioning(const struct logical_volume *lv);
+uint64_t get_thin_pool_max_metadata_size(struct cmd_context *cmd, struct profile *profile,
+					 thin_crop_metadata_t *crop);
+thin_crop_metadata_t get_thin_pool_crop_metadata(struct cmd_context *cmd,
+						 thin_crop_metadata_t crop,
+						 uint64_t metadata_size);
+uint64_t estimate_thin_pool_metadata_size(uint32_t data_extents, uint32_t extent_size, uint32_t chunk_size);
+
 /*
  * Begin skeleton for external LVM library
  */
@@ -538,5 +539,7 @@ void set_pv_devices(struct format_instance *fid, struct volume_group *vg);
 int get_visible_lvs_using_pv(struct cmd_context *cmd, struct volume_group *vg, struct device *dev,
                             struct dm_list *lvs_list);
 
+bool scan_text_mismatch(struct cmd_context *cmd, const char *vgname, const char *vgid);
+
 
 #endif
