@@ -77,6 +77,7 @@ int exec_cmd(struct cmd_context *cmd, const char *const argv[],
 
 	if (!pid) {
 		/* Child */
+		init_log_command(find_config_tree_bool(cmd, log_command_names_CFG, NULL), 0);
 		reset_locking();
 		/* FIXME Fix effect of reset_locking on cache then include this */
 		/* destroy_toolcontext(cmd); */
@@ -123,7 +124,7 @@ static int _reopen_fd_to_null(int fd)
 		return 0;
 	}
 
-	if (close(fd)) {
+	if ((null_fd != fd) && close(fd)) {
 		log_sys_error("close", "");
 		goto out;
 	}
@@ -194,6 +195,8 @@ FILE *pipe_open(struct cmd_context *cmd, const char *const argv[],
 	/* Parent -> reader */
 	if (close(pipefd[1 /*write*/])) {
 		log_sys_error("close", "STDOUT");
+		if (close(pipefd[0 /*read*/]))
+			log_sys_debug("close", "pipe[0]");
 		return NULL;
 	}
 
