@@ -127,6 +127,10 @@ static const char * const _blacklist_maps[] = {
 	"/libuuid.so.",		/* not using uuid during mlock (blkid) */
 	"/libz.so.",		/* not using during mlock (udev) */
 	"/libzstd.so.",		/* not using zstd during mlock (systemd) */
+	"/libjson-c.so.",	/* not using json during mlock (systemd) */
+	"/libkeyutils.so.",	/* not using keyutils during mlock (systemd) */
+	"/libcrypto.so.",	/* not using crypto during mlock (systemd) */
+	"/libnvme.so.",		/* not using nvme during mlock */
 	"/etc/selinux",		/* not using selinux during mlock */
 	/* "/libdevmapper-event.so" */
 };
@@ -353,7 +357,7 @@ static int _memlock_maps(struct cmd_context *cmd, lvmlock_t lock, size_t *mstats
 		if (!_maps_buffer || len >= _maps_len) {
 			if (_maps_buffer)
 				_maps_len *= 2;
-			if (!(line = realloc(_maps_buffer, _maps_len))) {
+			if (!_maps_len || !(line = realloc(_maps_buffer, _maps_len))) {
 				log_debug_mem("Allocation of maps buffer failed.");
 				return 0;
 			}
@@ -517,7 +521,7 @@ static void _restore_priority_if_possible(struct cmd_context *cmd)
 /* Stop memory getting swapped out */
 static void _lock_mem(struct cmd_context *cmd)
 {
-	if (!_size_stack || _size_malloc_tmp) {
+	if (!_size_stack || !_size_malloc_tmp) {
 		log_debug_mem("Skipping memory locking (reserved memory: "
 			      FMTsize_t "  stack: " FMTsize_t ").",
 			      _size_malloc_tmp, _size_stack);
@@ -564,7 +568,7 @@ static void _unlock_mem(struct cmd_context *cmd)
 {
 	size_t unlock_mstats = 0;
 
-	if (!_size_stack || _size_malloc_tmp) {
+	if (!_size_stack || !_size_malloc_tmp) {
 		log_debug_mem("Skipping memory unlocking (reserved memory: "
 			      FMTsize_t "  stack: " FMTsize_t ").",
 			      _size_malloc_tmp, _size_stack);
