@@ -17,7 +17,7 @@ SKIP_WITH_CLVMD=1
 
 . lib/inittest
 
-# Unsupported with valgrid testing
+# Unsupported with valgrind testing
 test "${LVM_VALGRIND:-0}" -eq 0 || skip "Unsupported with valgrind"
 
 # NOTE: Some tests, namely anything with vdo, and
@@ -31,6 +31,8 @@ aux extend_filter_LVMTEST
 # output
 aux prepare_profiles "lvmdbusd"
 
+# Keep generating test file within test dir
+export TMPDIR=$PWD
 aux prepare_lvmdbusd
 
 # Example for testing individual test:
