@@ -196,10 +196,10 @@ vgcreate $vg2 "$dev2"
 vgcreate $vg3 "$dev3"
 vgcreate $vg4 "$dev4"
 cp "$DF" "$ORIG"
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
-OPVID3=$(echo $(pvs --noheading -o uuid "$dev3") )
-OPVID4=$(echo $(pvs --noheading -o uuid "$dev4") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
+OPVID3="$(echo $(pvs --noheading -o uuid "$dev3") )"
+OPVID4="$(echo $(pvs --noheading -o uuid "$dev4") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 PVID3=${OPVID3//-/}
@@ -345,10 +345,10 @@ vgcreate $vg2 "$dev2"
 vgcreate $vg3 "$dev3"
 vgcreate $vg4 "$dev4"
 cp "$DF" "$ORIG"
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
-OPVID3=$(echo $(pvs --noheading -o uuid "$dev3") )
-OPVID4=$(echo $(pvs --noheading -o uuid "$dev4") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
+OPVID3="$(echo $(pvs --noheading -o uuid "$dev3") )"
+OPVID4="$(echo $(pvs --noheading -o uuid "$dev4") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 PVID3=${OPVID3//-/}
@@ -472,8 +472,8 @@ touch "$DF"
 vgcreate $vg1 "$dev1"
 vgcreate $vg4 "$dev4"
 cp "$DF" "$ORIG"
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID4=$(echo $(pvs --noheading -o uuid "$dev4") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID4="$(echo $(pvs --noheading -o uuid "$dev4") )"
 PVID1=${OPVID1//-/}
 PVID4=${OPVID4//-/}
 
@@ -551,7 +551,7 @@ rm "$DF"
 touch "$DF"
 vgcreate $vg2 "$dev2"
 cp "$DF" "$ORIG"
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
 PVID2=${OPVID2//-/}
 
 grep "$PVID2" "$DF" |tee out
@@ -601,10 +601,10 @@ grep "$SERIAL1" "$DF"
 grep "$SERIAL2" "$DF"
 grep "$SERIAL3" "$DF"
 grep "$SERIAL4" "$DF"
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
-OPVID3=$(echo $(pvs --noheading -o uuid "$dev3") )
-OPVID4=$(echo $(pvs --noheading -o uuid "$dev4") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
+OPVID3="$(echo $(pvs --noheading -o uuid "$dev3") )"
+OPVID4="$(echo $(pvs --noheading -o uuid "$dev4") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 PVID3=${OPVID3//-/}
@@ -665,10 +665,10 @@ vgcreate $vg2 "$dev2"
 vgcreate $vg3 "$dev3"
 vgcreate $vg4 "$dev4"
 cp "$DF" "$ORIG"
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
-OPVID3=$(echo $(pvs --noheading -o uuid "$dev3") )
-OPVID4=$(echo $(pvs --noheading -o uuid "$dev4") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
+OPVID3="$(echo $(pvs --noheading -o uuid "$dev3") )"
+OPVID4="$(echo $(pvs --noheading -o uuid "$dev4") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 PVID3=${OPVID3//-/}
@@ -730,9 +730,9 @@ vgcreate $vg1 "$dev1"
 vgcreate $vg2 "$dev2"
 vgcreate $vg3 "$dev3"
 cp "$DF" "$ORIG"
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
-OPVID3=$(echo $(pvs --noheading -o uuid "$dev3") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
+OPVID3="$(echo $(pvs --noheading -o uuid "$dev3") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 PVID3=${OPVID3//-/}
@@ -830,10 +830,10 @@ grep "$dev4" out
 grep "$dev2" out
 grep "$dev3" out
 
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
-OPVID3=$(echo $(pvs --noheading -o uuid "$dev3") )
-OPVID4=$(echo $(pvs --noheading -o uuid "$dev4") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
+OPVID3="$(echo $(pvs --noheading -o uuid "$dev3") )"
+OPVID4="$(echo $(pvs --noheading -o uuid "$dev4") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 PVID3=${OPVID3//-/}
@@ -867,8 +867,8 @@ lvmdevices --adddev "$dev3"
 cat "$DF"
 cp "$DF" "$ORIG"
 
-OPVID1=$(echo $(pvs --noheading -o uuid "$dev1") )
-OPVID2=$(echo $(pvs --noheading -o uuid "$dev2") )
+OPVID1="$(echo $(pvs --noheading -o uuid "$dev1") )"
+OPVID2="$(echo $(pvs --noheading -o uuid "$dev2") )"
 PVID1=${OPVID1//-/}
 PVID2=${OPVID2//-/}
 
