@@ -388,8 +388,8 @@ wait_md_create "$mddev"
 vgcreate $vg9 "$mddev"
 lvmdevices --adddev "$mddev" || true
 
-PVIDMD=$(pvs "$mddev" --noheading -o uuid | tr -d - | awk '{print $1}')
-BDEVMD=$(basename "$mddev")
+PVIDMD="$(pvs "$mddev" --noheading -o uuid | tr -d - | awk '{print $1}')"
+BDEVMD="$(basename "$mddev")"
 
 lvcreate -l1 -an -n $lv1 $vg9
 lvcreate -l1 -an -n $lv2 $vg9
