@@ -74,7 +74,7 @@ static const struct lv_type _lv_types[LVT_COUNT + 1] = {
 
 static const struct cmd_name cmd_names[CMD_COUNT + 1] = {
 #define cmd(a, b) { # b, a },
-#include "../include/cmds.h"
+#include "include/cmds.h"
 #undef cmd
 };
 
@@ -86,7 +86,7 @@ static const struct cmd_name cmd_names[CMD_COUNT + 1] = {
 #ifdef MAN_PAGE_GENERATOR
 
 static const struct command_name command_names[] = {
-#define xx(a, b, c...) { # a, b, c, NULL, a ## _COMMAND },
+#define xx(a, b, c...) { # a, b, NULL, c, a ## _COMMAND },
 #include "commands.h"
 #undef xx
 };
@@ -95,7 +95,7 @@ static struct command commands[COMMAND_COUNT];
 #else /* MAN_PAGE_GENERATOR */
 
 const struct command_name command_names[] = {
-#define xx(a, b, c...) { # a, b, c, a, a ## _COMMAND },
+#define xx(a, b, c...) { # a, b, a, c, a ## _COMMAND },
 #include "commands.h"
 #undef xx
 };
@@ -1370,7 +1370,7 @@ int define_commands(struct cmd_context *cmdtool, const char *run_name)
 					return 0;
 				}
 
-				snprintf(newdesc, newlen, "%s %s", cmd->desc, line_orig);
+				snprintf(newdesc, newlen, "%s%s", cmd->desc, line_orig);
 #ifdef MAN_PAGE_GENERATOR
 				free((void*)cmd->desc);
 #endif
@@ -1851,7 +1851,7 @@ void print_usage(struct command *cmd, int longhelp, int desc_first)
 			/*
 			 * Skip common lvm options in lvm_all which
 			 * are printed at the end under "Common options for lvm"
-			 * see print_common_options_lvm()
+			 * see print_usage_common_lvm()
 			 */
 
 			if (_is_lvm_all_opt(opt_enum))
@@ -1863,7 +1863,7 @@ void print_usage(struct command *cmd, int longhelp, int desc_first)
 			 * cname->common_options (options common
 			 * to all variants), which are printed at
 			 * the end under "Common options for command"
-			 * see print_common_options_cmd()
+			 * see print_usage_common_cmd()
 			 */
 
 			if (cna && (cna->variants > 1) && cna->common_options[opt_enum])
@@ -1891,7 +1891,7 @@ void print_usage(struct command *cmd, int longhelp, int desc_first)
 			/*
 			 * Skip common lvm options in lvm_all which
 			 * are printed at the end under "Common options for lvm"
-			 * see print_common_options_lvm()
+			 * see print_usage_common_lvm()
 			 */
 
 			if (_is_lvm_all_opt(opt_enum))
@@ -1903,7 +1903,7 @@ void print_usage(struct command *cmd, int longhelp, int desc_first)
 			 * cname->common_options (options common
 			 * to all variants), which are printed at
 			 * the end under "Common options for command"
-			 * see print_common_options_cmd()
+			 * see print_usage_common_cmd()
 			 */
 
 			if (cna && (cna->variants > 1) && cna->common_options[opt_enum])
@@ -2130,7 +2130,7 @@ void print_usage_notes(const struct command_name *cname)
 	       "capitalization, e.g. 'k' and 'K' both refer to 1024.\n\t"
 	       "The default input unit is specified by letter, followed by |UNIT.\n\t"
 	       "UNIT represents other possible input units: BbBsSkKmMgGtTpPeE.\n\t"
-	       "(This should not be confused with the output control --units, where\n\t"
-	       "capital letters mean multiple of 1000.)\n"
+	       "(This should not be confused with the output control --units,\n\t"
+	       "where capital letters mean multiple of 1000.)\n"
 	       "\n");
 }
