@@ -19,7 +19,6 @@
 #include "lib/metadata/metadata.h"
 #include "lib/label/hints.h"
 #include "lib/device/online.h"
-#include "lib/filters/filter.h"
 
 #include <dirent.h>
 
@@ -89,7 +88,46 @@ static int _pvscan_display_pv(struct cmd_context *cmd,
 		pv_len += suffix_len;
 	}
 
-	if (is_orphan(pv))
+	if (arg_is_set(cmd, allpvs_ARG)) {
+		struct device *dev = pv->dev;
+		if (!cmd->enable_devices_file) {
+			if (is_orphan(pv)) {
+				log_print_unless_silent("PV %-*s    %-*s",
+						pv_len, pvdevname,
+						params->vg_max_name_len, " ");
+			} else {
+				log_print_unless_silent("PV %-*s VG %-*s",
+						pv_len, pvdevname,
+						params->vg_max_name_len, pv_vg_name(pv));
+			}
+		} else if (!(dev->flags & DEV_MATCHED_USE_ID)) {
+			if (is_orphan(pv)) {
+				log_print_unless_silent("PV %-*s    %-*s %-10s %s",
+						pv_len, pvdevname,
+						params->vg_max_name_len, " ",
+						"-", "-");
+			} else {
+				log_print_unless_silent("PV %-*s VG %-*s %-10s %s",
+						pv_len, pvdevname,
+						params->vg_max_name_len, pv_vg_name(pv),
+						"-", "-");
+			}
+		} else {
+			if (is_orphan(pv)) {
+				log_print_unless_silent("PV %-*s    %-*s %-10s %s",
+						pv_len, pvdevname,
+						params->vg_max_name_len, " ",
+						idtype_to_str(dev->id->idtype),
+						dev->id->idname ?: "none");
+			} else {
+				log_print_unless_silent("PV %-*s VG %-*s %-10s %s",
+						pv_len, pvdevname,
+						params->vg_max_name_len, pv_vg_name(pv),
+						idtype_to_str(dev->id->idtype),
+						dev->id->idname ?: "none");
+			}
+		}
+	} else if (is_orphan(pv))
 		log_print_unless_silent("PV %-*s    %-*s %s [%s]",
 					pv_len, pvdevname,
 					params->vg_max_name_len, " ",
@@ -151,6 +189,11 @@ int pvscan_display_cmd(struct cmd_context *cmd, int argc, char **argv)
 			  arg_is_set(cmd, exported_ARG) ?
 			  "of exported volume group(s)" : "in no volume group");
 
+	if (arg_is_set(cmd, allpvs_ARG)) {
+		cmd->filter_deviceid_skip = 1;
+		cmd->use_hints = 0;
+	}
+
 	if (!(handle = init_processing_handle(cmd, NULL))) {
 		log_error("Failed to initialize processing handle.");
 		ret = ECMD_FAILED;
@@ -177,21 +220,6 @@ out:
 	return ret;
 }
 
-static void _lookup_file_remove(char *vgname)
-{
-	char path[PATH_MAX];
-
-	if (dm_snprintf(path, sizeof(path), "%s/%s", PVS_LOOKUP_DIR, vgname) < 0) {
-		log_error("Path %s/%s is too long.", PVS_LOOKUP_DIR, vgname);
-		return;
-	}
-
-	log_debug("Unlink pvs_lookup: %s", path);
-
-	if (unlink(path) && (errno != ENOENT))
-		log_sys_debug("unlink", path);
-}
-
 /*
  * When a device goes offline we only know its major:minor, not its PVID.
  * Since the dev isn't around, we can't read it to get its PVID, so we have to
@@ -200,15 +228,15 @@ static void _lookup_file_remove(char *vgname)
  * were created from.
  */
 
-static void _online_pvid_file_remove_devno(int major, int minor)
+static void _online_pvid_file_remove_devno(unsigned major, unsigned minor)
 {
 	char path[PATH_MAX];
 	char file_vgname[NAME_LEN];
 	DIR *dir;
 	struct dirent *de;
-	int file_major = 0, file_minor = 0;
+	unsigned file_major, file_minor;
 
-	log_debug("Remove pv online devno %d:%d", major, minor);
+	log_debug("Remove pv online devno %u:%u", major, minor);
 
 	if (!(dir = opendir(PVS_ONLINE_DIR)))
 		return;
@@ -233,7 +261,7 @@ static void _online_pvid_file_remove_devno(int major, int minor)
 
 			if (file_vgname[0]) {
 				online_vg_file_remove(file_vgname);
-				_lookup_file_remove(file_vgname);
+				online_lookup_file_remove(file_vgname);
 			}
 		}
 	}
@@ -317,7 +345,12 @@ static void _lookup_file_count_pvid_files(FILE *fp, const char *vgname, int *pvs
 
 	log_debug("checking all pvid files using lookup file for %s", vgname);
 
+	errno = 0;
 	rewind(fp);
+	if (errno) {
+		/* trace possible failure from rewind() */
+		log_sys_debug("rewind", vgname);
+	}
 
 	while (fgets(line, sizeof(line), fp)) {
 		memcpy(pvid, line, ID_LEN);
@@ -467,7 +500,7 @@ static int _pvscan_aa_single(struct cmd_context *cmd, const char *vg_name,
 
 	log_debug("pvscan autoactivating VG %s.", vg_name);
 
-	if (!vgchange_activate(cmd, vg, CHANGE_AAY, 1)) {
+	if (!vgchange_activate(cmd, vg, CHANGE_AAY, 1, NULL)) {
 		log_error_pvscan(cmd, "%s: autoactivation failed.", vg->name);
 		pp->activate_errors++;
 	}
@@ -519,7 +552,7 @@ static int _get_devs_from_saved_vg(struct cmd_context *cmd, const char *vgname,
 	struct volume_group *vg;
 	const char *name1, *name2;
 	dev_t devno;
-	int file_major = 0, file_minor = 0;
+	unsigned file_major = 0, file_minor = 0;
 
 	/*
 	 * We previously saved the metadata (as a struct vg) from the device
@@ -718,7 +751,7 @@ static int _pvscan_aa_quick(struct cmd_context *cmd, struct pvscan_aa_params *pp
 	 * devices used by the VG we read.
 	 */
 	dm_list_iterate_items(pvl, &vg->pvs) {
-		if (dev_in_device_list(pvl->pv->dev, &devs))
+		if (device_list_find_dev(&devs, pvl->pv->dev))
 			continue;
 		log_error_pvscan(cmd, "activation for VG %s found different devices.", vgname);
 		ret = ECMD_FAILED;
@@ -727,7 +760,7 @@ static int _pvscan_aa_quick(struct cmd_context *cmd, struct pvscan_aa_params *pp
 
 	log_debug("pvscan autoactivating VG %s.", vgname);
 
-	if (!vgchange_activate(cmd, vg, CHANGE_AAY, 1)) {
+	if (!vgchange_activate(cmd, vg, CHANGE_AAY, 1, NULL)) {
 		log_error_pvscan(cmd, "%s: autoactivation failed.", vg->name);
 		pp->activate_errors++;
 	}
@@ -818,49 +851,6 @@ out:
 	return ret;
 }
 
-/*
- * The optimization in which only the pvscan arg devname is added to dev-cache
- * does not work if there's an lvm.conf filter containing symlinks to the dev
- * like /dev/disk/by-id/lvm-pv-uuid-xyz entries.  A full dev_cache_scan will
- * associate the symlinks with the system dev name passed to pvscan, which lets
- * filter-regex match the devname with the symlink name in the filter.
- */
-static int _filter_uses_symlinks(struct cmd_context *cmd, int filter_cfg)
-{
-	const struct dm_config_node *cn;
-	const struct dm_config_value *cv;
-	const char *fname;
-
-	if ((cn = find_config_tree_array(cmd, filter_cfg, NULL))) {
-        	for (cv = cn->v; cv; cv = cv->next) {
-			if (cv->type != DM_CFG_STRING)
-				continue;
-			if (!cv->v.str)
-				continue;
-
-			fname = cv->v.str;
-
-			if (fname[0] != 'a')
-				continue;
-
-			if (strstr(fname, "/dev/disk/"))
-				return 1;
-			if (strstr(fname, "/dev/mapper/"))
-				return 1;
-
-			/* In case /dev/disk/by was omitted */
-			if (strstr(fname, "lvm-pv-uuid"))
-				return 1;
-			if (strstr(fname, "dm-uuid"))
-				return 1;
-			if (strstr(fname, "wwn-"))
-				return 1;
-		}
-	}
-
-	return 0;
-}
-
 struct pvscan_arg {
 	struct dm_list list;
 	const char *devname;
@@ -895,6 +885,11 @@ static int _get_args(struct cmd_context *cmd, int argc, char **argv,
 			continue;
 		}
 
+		if ((major < 0) || (minor < 0)) {
+			log_warn("WARNING: Invalid major:minor %d:%d, skipping.", major, minor);
+			continue;
+		}
+
 		if (!(arg = dm_pool_zalloc(cmd->mem, sizeof(*arg))))
 			return_0;
 		arg->devno = MKDEV(major, minor);
@@ -925,30 +920,6 @@ static int _get_args_devs(struct cmd_context *cmd, struct dm_list *pvscan_args,
 	struct pvscan_arg *arg;
 	struct device_list *devl;
 
-	/*
-	 * If no devices file is used, and lvm.conf filter is set to
-	 * accept /dev/disk/by-id/lvm-pv-uuid-xyz or another symlink,
-	 * but pvscan --cache is passed devname or major:minor, so
-	 * pvscan needs to match its arg device to the filter symlink.
-	 * setup_dev_in_dev_cache() adds /dev/sda2 to dev-cache which
-	 * does not match a symlink to /dev/sda2, so we need a full
-	 * dev_cache_scan that will associate all symlinks to sda2,
-	 * which allows filter-regex to work.  This case could be
-	 * optimized if needed by adding dev-cache entries for each
-	 * filter "a" entry (filter symlink patterns would still need
-	 * a full dev_cache_scan.)
-	 * (When no devices file is used and 69-dm-lvm.rules is
-	 * used which calls pvscan directly, symlinks may not
-	 * have been created by other rules when pvscan runs, so
-	 * the full dev_cache_scan may still not find them.)
-	 */
-	if (!cmd->enable_devices_file && !cmd->enable_devices_list &&
-	    (_filter_uses_symlinks(cmd, devices_filter_CFG) ||
-	     _filter_uses_symlinks(cmd, devices_global_filter_CFG))) {
-		log_print_pvscan(cmd, "finding all devices for filter symlinks.");
-		dev_cache_scan(cmd);
-	}
-
 	/* pass NULL filter when getting devs from dev-cache, filtering is done separately */
 
 	/* in common usage, no dev will be found for a devno */
@@ -957,8 +928,8 @@ static int _get_args_devs(struct cmd_context *cmd, struct dm_list *pvscan_args,
 		if (!arg->devname && !arg->devno)
 			return_0;
 		if (!(arg->dev = setup_dev_in_dev_cache(cmd, arg->devno, arg->devname))) {
-			log_error_pvscan(cmd, "No device set up for arg %s %d:%d",
-					 arg->devname ?: "", (int)MAJOR(arg->devno), (int)MINOR(arg->devno));
+			log_error_pvscan(cmd, "No device set up for arg %s %u:%u.",
+					 arg->devname ?: "", MAJOR(arg->devno), MINOR(arg->devno));
 		}
 	}
 
@@ -983,7 +954,7 @@ static void _set_pv_devices_online(struct cmd_context *cmd, struct volume_group
 	char pvid[ID_LEN+1] = { 0 };
 	struct pv_list *pvl;
 	struct device *dev;
-	int major, minor;
+	unsigned major, minor;
 	dev_t devno;
 
 	dm_list_iterate_items(pvl, &vg->pvs) {
@@ -1306,7 +1277,7 @@ static int _online_devs(struct cmd_context *cmd, int do_all, struct dm_list *pvs
 			 * device and using that to get the struct dev and dev_name.
 			 * The user could pass this list of devices to --devices
 			 * to optimize a subsequent command (activation) on the VG.
-			 * Just call set_pv_devices_online (if not done othewise)
+			 * Just call set_pv_devices_online (if not done otherwise)
 			 * since that finds the devs.
 			 */
 		}
@@ -1407,7 +1378,8 @@ static int _pvscan_cache_all(struct cmd_context *cmd, int argc, char **argv,
 	 * which we want 'pvscan --cache' to do, and that uses
 	 * info from lvmcache, e.g. duplicate pv info.
 	 */
-	lvmcache_label_scan(cmd);
+	if (!lvmcache_label_scan(cmd))
+		return_0;
 
 	cmd->pvscan_recreate_hints = 0;
 	cmd->use_hints = 0;
@@ -1436,27 +1408,6 @@ static int _pvscan_cache_all(struct cmd_context *cmd, int argc, char **argv,
 	return 1;
 }
 
-/*
- * If /dev/sda* of /dev/vda* is excluded by the devices file
- * it's usually a misconfiguration that prevents proper booting,
- * so make it a special case to give extra info to help debugging.
- */
-static void _warn_excluded_root(struct cmd_context *cmd, struct device *dev)
-{
-	struct dev_use *du;
-	const char *cur_idname;
-
-	if (!(du = get_du_for_devname(cmd, dev_name(dev)))) {
-		log_warn("WARNING: no autoactivation for %s: not found in system.devices.", dev_name(dev));
-		return;
-	}
-
-	cur_idname = device_id_system_read(cmd, dev, du->idtype);
-
-	log_warn("WARNING: no autoactivation for %s: system.devices %s current %s.",
-		 dev_name(dev), du->idname, cur_idname ?: "missing device id");
-}
-
 static int _pvscan_cache_args(struct cmd_context *cmd, int argc, char **argv,
 			      struct dm_list *complete_vgnames)
 {
@@ -1504,7 +1455,7 @@ static int _pvscan_cache_args(struct cmd_context *cmd, int argc, char **argv,
 	dm_list_iterate_items(arg, &pvscan_args) {
 		if (arg->dev || !arg->devno)
 			continue;
-		_online_pvid_file_remove_devno((int)MAJOR(arg->devno), (int)MINOR(arg->devno));
+		_online_pvid_file_remove_devno(MAJOR(arg->devno), MINOR(arg->devno));
 	}
 
 	/*
@@ -1544,35 +1495,66 @@ static int _pvscan_cache_args(struct cmd_context *cmd, int argc, char **argv,
 	 * If a match fails here do not exclude it, that will be done below by
 	 * passes_filter() which runs filter-deviceid. The
 	 * relax_deviceid_filter case needs to be able to work around
-	 * unmatching devs.
+	 * unmatching devs, or unmatching product_uuid/hostname which means
+	 * we can ignore the device ID and use any device with a PVID listed
+	 * in system.devices.
 	 */
 
 	if (cmd->enable_devices_file) {
 		dm_list_iterate_items(devl, &pvscan_devs)
 			device_ids_match_dev(cmd, devl->dev);
-
 	}
 	if (cmd->enable_devices_list)
 		device_ids_match_device_list(cmd);
 
-	if (cmd->enable_devices_file && device_ids_use_devname(cmd)) {
+	if (cmd->enable_devices_file &&
+	    (device_ids_use_devname(cmd) || cmd->device_ids_refresh_trigger)) {
 		relax_deviceid_filter = 1;
 		cmd->filter_deviceid_skip = 1;
 	}
 
 	cmd->filter_nodata_only = 1;
 
+	/*
+	 * Hack to handle regex filter that contains a symlink name for dev arg.
+	 * pvscan --cache <dev> is called by our udev rule at a time when the
+	 * symlinks for <dev> may not all be created yet (by other udev rules.)
+	 * The regex filter in lvm.conf may refer to <dev> using a symlink name,
+	 * so we need to know all the symlinks for <dev> in order for the filter
+	 * to work correctly. Scanning /dev with dev_cache_scan() would usually
+	 * find all the symlink names for <dev>, adding them to dev->aliases,
+	 * which would let the filter work, but all symlinks aren't created yet.
+	 * But, the DEVLINKS env var, set by udev, contains all the symlink
+	 * names for <dev> that have been or *will be* created. So, we add all
+	 * these symlink names to dev->aliases, as if we had found them in /dev.
+	 * This allows <dev> to be recognized by a regex filter containing a
+	 * symlink for <dev>. We have to tell filter-regex to not set the
+	 * preferred name for <dev> to a symlink name since the <dev> may not
+	 * be usable by that symlink name yet.
+	 */
+	if ((dm_list_size(&pvscan_devs) == 1) &&
+	    !cmd->enable_devices_file &&
+	    !cmd->enable_devices_list) {
+		char *env_str;
+		struct dm_list *env_aliases;
+		devl = dm_list_item(dm_list_first(&pvscan_devs), struct device_list);
+		if ((env_str = getenv("DEVLINKS"))) {
+			log_debug("Finding symlink names from DEVLINKS for filter regex.");
+			log_debug("DEVLINKS %s", env_str);
+			env_aliases = str_to_str_list(cmd->mem, env_str, " ", 0);
+			dm_list_splice(&devl->dev->aliases, env_aliases);
+		} else {
+			log_debug("Finding symlink names from /dev for filter regex.");
+			dev_cache_scan(cmd);
+		}
+		cmd->filter_regex_set_preferred_name_disable = 1;
+	}
+
 	dm_list_iterate_items_safe(devl, devl2, &pvscan_devs) {
 		if (!cmd->filter->passes_filter(cmd, cmd->filter, devl->dev, NULL)) {
 			log_print_pvscan(cmd, "%s excluded: %s.",
 					 dev_name(devl->dev), dev_filtered_reason(devl->dev));
 			dm_list_del(&devl->list);
-
-			/* Special case warning when probable root dev is missing from system.devices */
-			if ((devl->dev->filtered_flags & DEV_FILTERED_DEVICES_FILE) &&
-			    (!strncmp(dev_name(devl->dev), "/dev/sda", 8) ||
-			     !strncmp(dev_name(devl->dev), "/dev/vda", 8)))
-				_warn_excluded_root(cmd, devl->dev);
 		}
 	}
 
@@ -1644,7 +1626,7 @@ static int _pvscan_cache_args(struct cmd_context *cmd, int argc, char **argv,
 	/*
 	 * Scan devs to populate lvmcache info, which includes the mda info that's
 	 * needed to read vg metadata in the next step.  The _cached variant of
-	 * label_scan is used so the exsting bcache data from label_read_pvid above
+	 * label_scan is used so the existing bcache data from label_read_pvid above
 	 * can be reused (although more data may need to be read depending on how
 	 * much of the metadata was covered by reading the pvid.)
 	 */
@@ -1802,7 +1784,7 @@ int pvscan_cache_cmd(struct cmd_context *cmd, int argc, char **argv)
 int pvscan(struct cmd_context *cmd, int argc, char **argv)
 {
 	log_error(INTERNAL_ERROR "Missing function for command definition %d:%s.",
-		  cmd->command->command_index, cmd->command->command_id);
+		  cmd->command->command_index, command_enum(cmd->command->command_enum));
 	return ECMD_FAILED;
 }
 
